package net.ziemers.swxercise.db.dao.user;

import net.ziemers.swxercise.db.dao.GenericDao;
import net.ziemers.swxercise.lg.model.user.Role;

import javax.ejb.Stateless;

/**
 * Stellt Persistenz-Funktionalität im Kontext der Rollen- und Rechteverwaltung
 * zur Verfügung.
 *
 * Copyright (c) 2017 Dipl.-Inform. Thomas Ziemer
 *
 * This file is part of swXercise.
 *
 * swXercise is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * swXercise is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with swXercise.
 * If not, see <http://www.gnu.org/licenses/>.
 */
@Stateless
public class RoleDao extends GenericDao {

    /**
     * Findet eine {@link Role} aufgrund ihrer Id.
     *
     * @param id die Id der gewünschten Rolle
     * @return die Rolle oder <code>null</code>, falls es keine gibt.
     */
    public Role findById(final Long id) {
        Role role = null;

        try {
            // ermittelt den ersten Datensatz mit der gesuchten Id, auch wenn
            // er sich nicht im Persistence Context befindet
            role = (Role) entityManager.createNamedQuery("Role.findById")
                    .setParameter("id", id).getSingleResult();
        } catch (Exception e) {
            /* nix */
        }
        return role;
    }

    /**
     * Findet eine {@link Role} aufgrund ihres Rollennamens.
     *
     * @param name der name der gewünschten Rolle
     * @return den User oder null, falls es keinen gibt.
     */
    public Role findByName(final String name) {
        Role role = null;

        try {
            // ermittelt den ersten Datensatz mit dem gesuchten Rollennamen,
            // auch wenn er sich nicht im Persistence Context befindet
            role = (Role) entityManager.createNamedQuery("Role.findByName")
                    .setParameter("name", name).getSingleResult();
        } catch (Exception e) {
            /* nix */
        }
        return role;
    }

}
