package net.ziemers.swxercise.lg.user.service;

import net.ziemers.swxercise.lg.model.user.User;
import net.ziemers.swxercise.lg.user.dto.UserDto;

/**
 * Mapper zur Konvertierung des Kontexts einer {@link User}-Entity in ein {@link UserDto}.
 *
 * Copyright (c) 2017 Dipl.-Inform. Thomas Ziemer
 *
 * This file is part of swXercise.
 *
 * swXercise is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * swXercise is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with swXercise.
 * If not, see <http://www.gnu.org/licenses/>.
 */
public class EntityToUserDtoMapper {

    /**
     * Erzeugt ein Data Transfer Object aus dem übergebenem Kontext.
     *
     * @param ctx der Kontext
     * @return das erzeugte DTO.
     */
    public UserDto map(EntityToUserDtoContext ctx) {
        final UserDto dto = new UserDto();

        if (ctx.user != null) {
          mapUser(ctx, dto);
          mapFirstname(ctx, dto);
          mapLastname(ctx, dto);
        }
        return dto;
    }

    private void mapUser(EntityToUserDtoContext ctx, UserDto dto) {
        dto.setUser(ctx.user);
    }

    private void mapFirstname(EntityToUserDtoContext ctx, UserDto dto) {
        dto.setFirstname(ctx.user.getFirstname());
    }

    private void mapLastname(EntityToUserDtoContext ctx, UserDto dto) {
        dto.setLastname(ctx.user.getLastname());
    }

}
