package net.ziemers.swxercise.lg.user.service;

import net.ziemers.swxercise.lg.user.dto.UserDto;

import javax.ejb.Stateless;

/**
 * Mapper zur Konvertierung des Kontexts eines {@link UserDto}s in eine User-Entity.
 *
 * Copyright (c) 2017 Dipl.-Inform. Thomas Ziemer
 *
 * This file is part of swXercise.
 *
 * swXercise is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * swXercise is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with swXercise.
 * If not, see <http://www.gnu.org/licenses/>.
 */
@Stateless
public class UserDtoToEntityMapper {

    /**
     * Überträgt die Eigenschaften aus dem UserDto sowie dem zusätzlichen Kontext in die Zielentitäten.
     * Anzumerken ist, dass der Benutzername einer existierenden Profile-Entität niemals aktualisiert
     * wird.
     *
     * @param ctx der Kontext mit den Eigenschaften und der Zielentität
     * @return den Kontext.
     */
    public UserDtoToEntityContext map(UserDtoToEntityContext ctx) {
        // ins User-Objekt mappen
        ctx.user.setFirstname(ctx.dto.getFirstname());
        ctx.user.setLastname(ctx.dto.getLastname());

        // ins Profile-Objekt mappen, falls gegeben
        if (ctx.profile != null) {
            ctx.user.setProfile(ctx.profile);

            // eventuell ein neues Kenntwort ins Profile-Objekt mappen
            if(ctx.dto.getPassword().length() > 0) {
                ctx.profile.setPassword(ctx.dto.getPassword());
            }
            ctx.profile.setMailaddress(ctx.dto.getMailaddress());
        }

        // ins Address-Objekt mappen, falls gegeben
        if (ctx.address != null) {
            ctx.user.setAddress(ctx.address);
        }
        return ctx;
    }

}
