package net.ziemers.swxercise.ui.utils;

import net.ziemers.swxercise.ui.enums.ResponseState;

/**
 * Die Ergebnis-Klasse für die meisten REST-API-Methoden.
 *
 * Copyright (c) 2017 Dipl.-Inform. Thomas Ziemer
 *
 * This file is part of swXercise.
 *
 * swXercise is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * swXercise is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with swXercise.
 * If not, see <http://www.gnu.org/licenses/>.
 */
public class RestResponse {

    private ResponseState responseState;

    private String message;

    public RestResponse() {
        this(ResponseState.SUCCESS);
    }

    public RestResponse(final ResponseState responseState) {
        this.responseState = responseState;        
    }

    public RestResponse(final ResponseState responseState, final String message) {
        this(responseState);
        this.message = message;
    }

    public RestResponse(final String message) {
        this();
        this.message = message;
    }

    public int getResponseCode() {
        return responseState.getResponseCode();
    }

    public String getResponseText() {
        return responseState.getResponseText();
    }

    public String getMessage() { return message; }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        RestResponse that = (RestResponse) o;

        return responseState == that.responseState;
    }

}

