package net.ziemers.swxercise.lg.testdatabuilder;

import javax.persistence.EntityManager;

/**
 * Copyright (c) 2017 Dipl.-Inform. Thomas Ziemer
 *
 * This file is part of swXercise.
 *
 * swXercise is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * swXercise is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with swXercise.
 * If not, see <http://www.gnu.org/licenses/>.
 */
public abstract class AbstractTestDataBuilder<T> {

    private static int count = 0;

    private EntityManager entityManager;

    /**
     * Creates an new TestdataBuilder with persistence.
     *
     * @param entityManager
     */
    public AbstractTestDataBuilder(final EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    /**
     * Creates an new TestdataBuilder without persistence.
     */
    public AbstractTestDataBuilder() {
    }

    public abstract T build();

    /**
     * Returns the EntityManager or null.
     *
     * @return {@link EntityManager} or null
     */
    protected final EntityManager getEntityManager() {
        return entityManager;
    }

    /**
     * Ensure the TestdataBuilder is constructed with a
     * {@link EntityManager}
     *
     * @throws IllegalStateException
     *             if the TestdataBuilder is constructed without a
     *             {@link EntityManager}
     */
    protected final void ensureEntityManager() {
        if (entityManager == null) {
            throw new IllegalStateException("Cannot persist w/o entity manager");
        }
    }

    /**
     * {@inheritDoc} Executed within a new transaction.
     *
     * @throws IllegalStateException
     *             if the TestdataBuilder is constructed without a
     *             {@link EntityManager}
     */
    public final T buildAndSave() {
        ensureEntityManager();
        try {
            final T obj = build();
            entityManager.persist(obj);
            return obj;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Returns an integer value from an static counter.
     *
     * @return value of the static counter.
     */
    protected final int getId() {
        return count++;
    }

}

